@@ -41,7 +41,7 @@ static GList *window_list = NULL;
 
 /* what can be dragged in us... */
 enum {
-        TARGET_URI_LIST,
+        TARGET_URI_LIST
 };
 
 static void
@@ -257,14 +257,29 @@ ggv_window_drag_data_received(GtkWidget *widget,
 #ifdef _DEBUG
                         g_message("URI %s\n", *uri);
 #endif /* _DEBUG */
-                        if(newwin == NULL) {
+                        if(newwin == NULL)
                                 newwin = ggv_window_new();
-                                gtk_widget_show(newwin);
+                        if(ggv_window_load(GGV_WINDOW(newwin), (*uri))) {
+                                if(newwin != GTK_WIDGET(win))
+                                        gtk_widget_show(newwin);
+                                newwin = NULL;
+                        }
+                        else {
+                                GtkWidget *dlg;
+                                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                                             GTK_DIALOG_MODAL,
+                                                             GTK_MESSAGE_ERROR,
+                                                             GTK_BUTTONS_OK,
+                                                             _("Unable to load file:\n%s"),
+                                                             *uri);
+                                gtk_widget_show(dlg);
+                                gtk_dialog_run(GTK_DIALOG(dlg));
+                                gtk_widget_destroy(dlg);
                         }
-                        ggv_window_load(GGV_WINDOW(newwin), (*uri));
-                        newwin = NULL;
                         uri++;
                 }
+                if(newwin != NULL && newwin != GTK_WIDGET(win))
+                        ggv_window_close(GGV_WINDOW(newwin));
                 g_strfreev(win->uris_to_open);
                 win->uris_to_open = NULL;
         }
@@ -304,10 +319,10 @@ verb_FileOpen(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
         EggRecentItem *item;
         EggRecentModel *recent;
-
+        GSList *uris;
         GgvWindow *win = GGV_WINDOW(data);
-        gchar *fname = NULL, *escaped_fname;
 	GtkWidget *chooser;
+        GtkWidget *newwin;
 
 	chooser = gtk_file_chooser_dialog_new(_("Select a PostScript document"),
 					      GTK_WINDOW(win),
@@ -315,48 +330,71 @@ verb_FileOpen(BonoboUIComponent *uic, gpointer data, const char *cname)
 					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
 					      NULL);
+        gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(chooser), TRUE);
 	gtk_dialog_set_default_response(GTK_DIALOG(chooser), GTK_RESPONSE_OK);
 	gtk_window_set_default_size(GTK_WINDOW(chooser), 600, 400);
+
+#if 0
+        /* this causes unexplained crashes! */
 	if (win->filename)
 		gtk_file_chooser_set_uri(GTK_FILE_CHOOSER(chooser),
 					 win->filename);
+#endif
 
-	if (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_OK)
-		fname = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(chooser));
+        do {
+                uris = NULL;
+                if (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_OK)
+                        uris = gtk_file_chooser_get_uris
+                                (GTK_FILE_CHOOSER(chooser));
+                else
+                        break;
+        } while(NULL == uris);
 
 	gtk_widget_destroy(chooser);
 
-        if(fname) {
-                if(win->page_count > 0) {
-                        GtkWidget *newwin;
+        if(uris == NULL)
+                return;
 
+        if(win->page_count > 0)
+                newwin = NULL;
+        else
+                newwin = GTK_WIDGET(win);
+        while(uris) {
+                if(NULL == newwin)
                         newwin = ggv_window_new();
-                        if(NULL == newwin) {
-                                g_free(fname);
-                                return;
-                        }
-                        gtk_widget_show(newwin);
-                        win = GGV_WINDOW(newwin);
-                }
-                if(!ggv_window_load(win, fname)) {
+                if(!ggv_window_load(GGV_WINDOW(newwin),
+                                    (gchar *)uris->data)) {
                         GtkWidget *dlg;
-                        dlg = gtk_message_dialog_new(GTK_WINDOW(win),
-                                                     GTK_DIALOG_MODAL,
-                                                     GTK_MESSAGE_ERROR,
-                                                     GTK_BUTTONS_OK,
-                                                     _("Unable to load file:\n%s"),
-                                                     fname);
+                        dlg = gtk_message_dialog_new
+                                (GTK_WINDOW(win),
+                                 GTK_DIALOG_MODAL,
+                                 GTK_MESSAGE_ERROR,
+                                 GTK_BUTTONS_OK,
+                                 _("Unable to load file:\n%s"),
+                                 (gchar *)uris->data);
                         gtk_widget_show(dlg);
                         gtk_dialog_run(GTK_DIALOG(dlg));
                         gtk_widget_destroy(dlg);
                 }
-		recent = ggv_recent_get_model ();
-		item = egg_recent_item_new_from_uri (fname);
-		egg_recent_model_add_full (recent, item);
-		egg_recent_item_unref (item);
-
-                g_free(fname);
+                else {
+                        if(GTK_WIDGET(win) != newwin)
+                                gtk_widget_show(newwin);
+                        recent = ggv_recent_get_model ();
+                        item = egg_recent_item_new_from_uri
+                                ((gchar *)uris->data);
+                        egg_recent_item_set_mime_type
+                                (item, "application/postscript");
+                        egg_recent_item_add_group(item, "ggv");
+                        egg_recent_model_add_full (recent, item);
+                        egg_recent_item_unref (item);
+                        newwin = NULL;
+                }
+                g_free(uris->data);
+                uris = uris->next;
         }
+        if(NULL != newwin && GTK_WIDGET(win) != newwin)
+                ggv_window_close(GGV_WINDOW(newwin));
+        g_slist_free(uris);
 }
 
 static void
@@ -541,18 +579,35 @@ verb_DnDNewWindow(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 {
         GgvWindow *win = GGV_WINDOW(user_data);
         gchar **uri;
-        GtkWidget *newwin;
+        GtkWidget *newwin = NULL;
 
         uri = win->uris_to_open;
         while(*uri && **uri != '\0') {
 #ifdef _DEBUG
                 g_message("URI %s", *uri);
 #endif /* _DEBUG */
-                newwin = ggv_window_new();
-                gtk_widget_show(newwin);
-                ggv_window_load(GGV_WINDOW(newwin), (*uri));
+                if(NULL == newwin)
+                        newwin = ggv_window_new();
+                if(ggv_window_load(GGV_WINDOW(newwin), (*uri))) {
+                        gtk_widget_show(newwin);
+                        newwin = NULL;
+                }
+                else {
+                        GtkWidget *dlg;
+                        dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                                     GTK_DIALOG_MODAL,
+                                                     GTK_MESSAGE_ERROR,
+                                                     GTK_BUTTONS_OK,
+                                                     _("Unable to load file:\n%s"),
+                                                     *uri);
+                        gtk_widget_show(dlg);
+                        gtk_dialog_run(GTK_DIALOG(dlg));
+                        gtk_widget_destroy(dlg);
+                }
                 uri++;
         }
+        if(NULL != newwin)
+                ggv_window_close(GGV_WINDOW(newwin));
         g_strfreev(win->uris_to_open);
         win->uris_to_open = NULL;
 }
@@ -570,14 +625,29 @@ verb_DnDThisWindow(BonoboUIComponent *uic, gpointer user_data, const char *cname
 #ifdef _DEBUG
                 g_message("URI %s", *uri);
 #endif /* _DEBUG */
-                if(newwin == NULL) {
+                if(newwin == NULL)
                         newwin = ggv_window_new();
-                        gtk_widget_show(newwin);
+                if(ggv_window_load(GGV_WINDOW(newwin), (*uri))) {
+                        if(newwin != GTK_WIDGET(win))
+                                gtk_widget_show(newwin);
+                        newwin = NULL;
+                }
+                else {
+                        GtkWidget *dlg;
+                        dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                                     GTK_DIALOG_MODAL,
+                                                     GTK_MESSAGE_ERROR,
+                                                     GTK_BUTTONS_OK,
+                                                     _("Unable to load file:\n%s"),
+                                                     *uri);
+                        gtk_widget_show(dlg);
+                        gtk_dialog_run(GTK_DIALOG(dlg));
+                        gtk_widget_destroy(dlg);
                 }
-                ggv_window_load(GGV_WINDOW(newwin), (*uri));
-                newwin = NULL;
                 uri++;
         }
+        if(NULL != newwin && newwin != GTK_WIDGET(win))
+                ggv_window_close(GGV_WINDOW(newwin));
         g_strfreev(win->uris_to_open);
         win->uris_to_open = NULL;	
 }
@@ -620,7 +690,7 @@ static void
 ggv_window_destroy(GtkObject *object)
 {
         GgvWindow *win;
-        gint i;
+        gint i, w, h;
 
         g_return_if_fail(object != NULL);
         g_return_if_fail(GGV_IS_WINDOW(object));
@@ -658,8 +728,12 @@ ggv_window_destroy(GtkObject *object)
 
         window_list = g_list_remove(window_list, win);
 
-        ggv_get_window_size(GTK_WIDGET(object),
-                            &ggv_default_width, &ggv_default_height);
+        
+        ggv_get_window_size(GTK_WIDGET(object), &w, &h);
+        if(w > 1 && h > 1) {
+                ggv_default_width = w;
+                ggv_default_height = h;
+        }
 
         if(GTK_OBJECT_CLASS(parent_class)->destroy)
                 GTK_OBJECT_CLASS(parent_class)->destroy(object);
@@ -676,8 +750,9 @@ static void
 prefs_changed(GConfClient* client, guint cnxn_id,
               GConfEntry *entry, gpointer user_data)
 {
+        GgvWindow *window = GGV_WINDOW(user_data);
+
         if(!strcmp(entry->key, "/apps/ggv/layout/showtoolbar")) {
-                GgvWindow *window = GGV_WINDOW(user_data);
                 gboolean state_f = gconf_value_get_bool(entry->value);
                 if(window->show_toolbar != state_f) {
                         window->show_toolbar = state_f;
@@ -688,7 +763,6 @@ prefs_changed(GConfClient* client, guint cnxn_id,
                 }
         }
         else if(!strcmp(entry->key, "/apps/ggv/layout/showmenubar")) {
-                GgvWindow *window = GGV_WINDOW(user_data);
                 gboolean state_f = gconf_value_get_bool(entry->value);
                 if(window->show_menus != state_f) {
                         window->show_menus = state_f;
@@ -699,7 +773,6 @@ prefs_changed(GConfClient* client, guint cnxn_id,
                 }
         }
         else if(!strcmp(entry->key, "/apps/ggv/layout/showpanel")) {
-                GgvWindow *window = GGV_WINDOW(user_data);
                 gboolean state_f = gconf_value_get_bool(entry->value);
                 if(window->show_sidebar != state_f) {
                         window->show_sidebar = state_f;
@@ -710,7 +783,6 @@ prefs_changed(GConfClient* client, guint cnxn_id,
                 }
         }
         else if(!strcmp(entry->key, "/apps/ggv/layout/showstatusbar")) {
-                GgvWindow *window = GGV_WINDOW(user_data);
                 gboolean state_f = gconf_value_get_bool(entry->value);
                 if(window->show_statusbar != state_f) {
                         window->show_statusbar = state_f;
@@ -722,6 +794,12 @@ prefs_changed(GConfClient* client, guint cnxn_id,
                                 gtk_widget_hide(window->statusbar);
                 }
         }
+        else if(!strcmp(entry->key, "/apps/ggv/layout/rightpanel")) {
+#if 0
+                bonobo_control_frame_control_deactivate(window->ctlframe);
+                bonobo_control_frame_control_activate(window->ctlframe);
+#endif
+        }
 }
 
 static void
@@ -844,16 +922,29 @@ ggv_file_open_recent (EggRecentView *view, EggRecentItem *item, gpointer data)
                                                                                                                              
         if(win->page_count <= 0)
                 newwin = GTK_WIDGET (win);
-        else
-                newwin = NULL;
-                                                                                                                             
-        if (newwin == NULL) {
+        else {
                 newwin = ggv_window_new ();
-                gtk_widget_show (GTK_WIDGET (newwin));
         }
                                                                                                                              
-        ggv_window_load (GGV_WINDOW (newwin), uri_utf8);
-        newwin = NULL;
+        if(ggv_window_load (GGV_WINDOW (newwin), uri_utf8)) {
+                if(newwin != GTK_WIDGET(win))
+                        gtk_widget_show(newwin);
+        }
+        else {
+                GtkWidget *dlg;
+                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                             GTK_DIALOG_MODAL,
+                                             GTK_MESSAGE_ERROR,
+                                             GTK_BUTTONS_OK,
+                                             _("Unable to load file:\n%s"),
+                                             uri_utf8);
+                gtk_widget_show(dlg);
+                gtk_dialog_run(GTK_DIALOG(dlg));
+                gtk_widget_destroy(dlg);
+                if(newwin != GTK_WIDGET(win))
+                        ggv_window_close(GGV_WINDOW(newwin));
+        }
+
                                                                                                                              
         return TRUE;
 }
@@ -873,7 +964,7 @@ ggv_add_open_button (GgvWindow *win, BonoboUIComponent *ui_component,
                            gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT));
                                                                                                                              
         gtk_widget_show_all (GTK_WIDGET (button));
-                                                                                                                             
+
         model = ggv_recent_get_model ();
                                                                                                                              
         menu = gtk_menu_new ();
@@ -1075,6 +1166,7 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
         Bonobo_PersistFile pf;
         gchar *title, *unescaped, *utf8_title, *_filename;
         GnomeVFSFileInfo info;
+	Bonobo_UIContainer container;
 
         g_return_val_if_fail(win != NULL, FALSE);
         g_return_val_if_fail(GGV_IS_WINDOW(win), FALSE);
@@ -1087,14 +1179,27 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
                                                     GNOME_VFS_FILE_INFO_FOLLOW_LINKS)) {
                 win->mtime = info.mtime;
         }
-        else
+        else {
+                GNOME_GGV_PostScriptView view;
+
+                CORBA_exception_init(&ev);
+                view = Bonobo_Unknown_queryInterface
+                        (win->control, "IDL:GNOME/GGV/PostScriptView:1.0",
+                         &ev);
+                if(!BONOBO_EX(&ev) && view != CORBA_OBJECT_NIL) {
+                        GNOME_GGV_PostScriptView_close(view, &ev);
+                }
+                CORBA_exception_free(&ev);
+                ggv_window_set_watch_file(win, ggv_watch_doc);
                 return FALSE;
+        }
 
         CORBA_exception_init(&ev);
         pf = Bonobo_Unknown_queryInterface(win->control,
                                            "IDL:Bonobo/PersistFile:1.0", &ev);
         if(BONOBO_EX(&ev) || pf == CORBA_OBJECT_NIL) {
                 CORBA_exception_free(&ev);
+                ggv_window_set_watch_file(win, ggv_watch_doc);
                 return FALSE;
         }
 
@@ -1116,14 +1221,18 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
 
         if(BONOBO_EX(&ev)) {
                 CORBA_exception_free(&ev);
+                ggv_window_set_watch_file(win, ggv_watch_doc);
                 return FALSE;
         }
         CORBA_exception_free(&ev);
 
-        bonobo_ui_component_set_prop
-                (win->uic, "/commands/FileReload", "sensitive",
-                 (bonobo_pbclient_get_long(win->pb, "page_count", NULL) > 0)?"1":"0",
-                 NULL);
+        win->page_count = bonobo_pbclient_get_long(win->pb, "page_count", NULL);
+
+	container = bonobo_ui_component_get_container (win->uic);
+	if (container != CORBA_OBJECT_NIL)
+                bonobo_ui_component_set_prop
+                        (win->uic, "/commands/FileReload", "sensitive",
+                         (win->page_count > 0)?"1":"0", NULL);
 
         ggv_window_set_watch_file(win, ggv_watch_doc);
 
@@ -1154,7 +1263,18 @@ void
 ggv_window_reload(GgvWindow *win)
 {
         gint page = win->current_page;  
-        ggv_window_load(win, win->filename);
+        if(!ggv_window_load(win, win->filename)) {
+                GtkWidget *dlg;
+                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                             GTK_DIALOG_MODAL,
+                                             GTK_MESSAGE_ERROR,
+                                             GTK_BUTTONS_OK,
+                                             _("Unable to load file:\n%s"),
+                                             win->filename);
+                gtk_widget_show(dlg);
+                gtk_dialog_run(GTK_DIALOG(dlg));
+                gtk_widget_destroy(dlg);
+        }
         bonobo_pbclient_set_long(win->pb, "page", page, NULL);
 }
 
@@ -1184,7 +1304,9 @@ timer_callback (gpointer data)
 	if ((info.mtime != win->mtime) &&
 	    (time (NULL) - info.mtime >= GGV_WATCH_TIMEOUT) &&
             (NULL != win->control)) {
-                ggv_window_reload(win);
+                gint page = win->current_page;  
+                ggv_window_load(win, win->filename);
+                bonobo_pbclient_set_long(win->pb, "page", page, NULL);
                 win->mtime = info.mtime;
         }
 
