@@ -112,8 +112,10 @@ The DONE message indicates that ghostscript has finished processing.
 #include <stdlib.h>
 #include <signal.h>
 #include <gtk/gtk.h>
+#include <gtk/gtkobject.h>
 #include <gdk/gdkprivate.h>
 #include <gdk/gdkx.h>
+#include <gdk/gdk.h>
 #ifdef  HAVE_XINERAMA
 #  include <gdk/gdkx.h>
 #  include <X11/extensions/Xinerama.h>
@@ -1768,11 +1770,10 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
 		}
 
                 /* we grab the vital statistics!!! */
-		gs->doc = psscan(gs->gs_psfile, gs->respect_eof);
+		gs->doc = psscan(gs->gs_psfile, gs->respect_eof, fname);
 
                 if (gs->doc == NULL) {
                         /* File does not seem to be a Postscript one */
-                        /* We have to report error to the reader, do we do it here */
 			gchar buf[1024];
 			g_snprintf (buf, 1024,
 				    _("Error while scanning file %s\n"), fname);
